﻿/*
NAME: Preloader
VERSION: 0.3 - BETA
ACTIONSCRIPT: 3.0
BY: Donovan Hubbard, http://donovanh.com
DESCRIPTION: Tracks the loading of objcts into flash. Allows the user to assign custom function to execute while
items are loading and another function for when every item is finished loading.

Some of the tools in this class:
 *trackLoading = Assigns an object for the preloader to watch. This may be a Sound, Loader, MovieClip, or Stage.
 				When an object is set to be tracked the preloader will continue to track that object, for example
				trackLoading(My_MC) would track the progress of any item being loaded into My_MC.
 *onLoaded [get/set] = This is the function that will get called when the preloader percentage = 1 (100%)
 *whileLoading [get/set] = This function gets called continually while the preloader is less than 1 (100%)
 				Common use for this would be to retreive and track the Preloader data 
 *getPreloaderInfo = returns an array as fallows [percent, all bytes loaded, all total bytes, all objects loading].

Known Issues:
 *Sound is not picked up by default when tracking the stage. All sound must be manually added to the tracking list.
 
*/
package
{
	import flash.net.*;
	import flash.display.*;
	import flash.events.*;
	import flash.utils.*;
	import flash.media.*;
	public class Preloader
	{
		public static var percent:Number = 0;
		public static var loaded:Number = 0;
		public static var total:Number = 0;
		public static var _loaderList:Array = new Array();	
		public static var preloaderInfo:Object = new Object();
		private static var finishedLoadingFunction:Function = new Function();
		private static var whileLoadingFunction:Function = new Function();
		public function Preloader(list:Array)
		{
			if(percent==1)
			{
				_loaderList = [];
				percent = preloaderInfo._percent = 0;
				total = preloaderInfo._bytesTotal = 0;
				loaded = preloaderInfo._bytesLoaded = 0;
				finishedLoadingFunction();
				return;
			}
			var allBL:Number=0;
			var allBT:Number=0;
			for(var a:int;a<list.length;a++)
			{
				if(list[a] is Stage)
				{
					allBL += list[a].getChildAt(0).framesLoaded;
					allBT += list[a].getChildAt(0).totalFrames;
				}else{
					allBL += list[a].bytesLoaded;
					allBT += list[a].bytesTotal;
				}
			}
			loaded = preloaderInfo._bytesLoaded = allBL;
			total = preloaderInfo._bytesTotal = allBT;
			percent = preloaderInfo._percent = allBL/allBT;
			whileLoadingFunction();
		}
		private static function dtrace(t:*)
		{
			trace("Preloader: "+t);
		}
		private static function etrace(t:*)
		{
			dtrace("ERROR! :"+t)
		}
		public static function get onLoaded():Function
		{
			return finishedLoadingFunction;
		}
		public static function set onLoaded(funk:Function)
		{
			finishedLoadingFunction = funk;
		}
		public static function get whileLoading():Function
		{
			return whileLoadingFunction;
		}
		public static function set whileLoading(funk:Function)
		{
			whileLoadingFunction = funk;
		}
		public static function getPreloaderInfo():Array
		{
			var plInfo:Array =  new Array();
			plInfo.push("percentage[0-1] = "+percent);
			plInfo.push(" loaded bytes = "+loaded);
			plInfo.push(" total bytes = "+total);
			plInfo.push(" objects being loaded = "+_loaderList);
			return plInfo;
		}
		private static function update(pe:ProgressEvent):Preloader
		{
			var targ:* = pe.target;
			return new Preloader(_loaderList);
		}
		private static function done(e:Event):Preloader
		{
			
			var targ:* = e.target;
			return new Preloader(_loaderList);
		}
		public static function trackLoading(obj:*)
		{
			switch(true)
			{
				case obj is Loader:
					obj.contentLoaderInfo.addEventListener(ProgressEvent.PROGRESS, update);
					obj.contentLoaderInfo.addEventListener(Event.COMPLETE, done);
					_loaderList.push(obj.contentLoaderInfo);
				break;
				case obj is Sound:
					obj.addEventListener(ProgressEvent.PROGRESS, update);
					obj.addEventListener(Event.COMPLETE, done);
					_loaderList.push(obj);
				break;
				case obj is MovieClip:
					obj.addEventListener(Event.ADDED, checkAdded);
					obj.addEventListener(Event.OPEN, checkAdded);
					obj.addEventListener(Event.CONNECT, checkAdded);
					obj.addEventListener(Event.ADDED_TO_STAGE, checkAdded);
					var loaders:Array = findLoadersInMC(obj);
					for(var i:int;i<loaders.length;i++)
					{
						trackLoading(loaders[i]);
					}
				break;
				case  obj is Stage:
					_loaderList.push(obj);
					obj.addEventListener(ProgressEvent.PROGRESS, update);
					obj.addEventListener(Event.ENTER_FRAME, done);
					obj.addEventListener(Event.CHANGE, done);
					obj.addEventListener(Event.ADDED, checkAdded);
					obj.addEventListener(Event.OPEN, checkAdded);
					obj.addEventListener(Event.CONNECT, checkAdded);
					obj.addEventListener(Event.ADDED_TO_STAGE, checkAdded);
					dtrace("stage is a go!");
				break;
				default:
					etrace(obj+" cannot be tracked with the Preloader class and will not be added to the loading list.");
				break;
			}
		}
		private static function checkAdded(e:Event)
		{
			var loader:* = e.target;
			dtrace("checkAdded: "+loader);
			if((loader is Loader)||(loader is Sound)||(loader is MovieClip))
			{
				trackLoading(loader);
			}
		}
		private static function findLoaderInObj(obj:Object):Array
		{
			var loaders:Array = new Array();
			for(var i:int;i<obj.numChildren;i++)
			{
				var nxtKid:* = obj.getChildAt(0);
				if(nxtKid is MovieClip)
				{
					loaders.concat(findLoadersInMC);
				}
				if(nxtKid is Loader)
				{
					loaders.push(nxtKid);
				}
			}
			return loaders;
		}
		private static function findLoadersInMC(mc:MovieClip):Array
		{
			var loaders:Array = new Array();
			for(var i:int;i<mc.numChildren;i++)
			{
				var nxtKid:* = mc.getChildAt(0);
				if(nxtKid is MovieClip)
				{
					loaders.concat(findLoadersInMC);
				}
				if(nxtKid is Loader)
				{
					loaders.push(nxtKid);
				}
			}
			return loaders;
		}
	}
}